#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

class Tracking(ALModule):

    def __init__(self, ip, port):

        self.port = 9559
        self.ip = ip

        self.name = "FaceTrackingEventListener"
        proxy_name = "ALTracker"

        self.tracker = ALProxy("ALTracker",self.ip, self.port)
        self.memory = ALProxy("ALMemory",self.ip, self.port)
        self.soundLocation = ALProxy("ALSoundLocalization",self.ip, self.port)
        self.targetName = "People"
        self.distanceX = 0.0
        self.distanceY = 0.0
        self.angleWz = 0.0
        self.soundDistance = 0.5
        self.thresholdX = 0.0
        self.thresholdY = 0.0
        self.thresholdWz = 0.0
        self.effector = "None"
        self.subscribeDone = False
        self.isRunning = False
        
        self.distance = 0.3
        self.soundDistance = 0.5
        self.confidence = 0.4
        self.sensitivity = 0.9
        self.memory.subscribeToEvent("ALTracker/ActiveTargetChanged", self.name, "onTargetChanged")
        print ( proxy_name + " success")
        self.running = False


    def onRun(self, input_ = "People", parameters = {"use_body":"Head", "use_arms":"None"}, parallel = False):

        if self.running == True:
            self.onStop()
            print("stoping last tracking and starting new one")
            self.onStart(input_, parameters, parallel)
        else:
            self.onStart(input_, parameters, parallel)
            
        #else:
            #self.onStop()


    def use_arms(self,parameters):

        value = "None"

        if parameters["LeftArm"] == True and parameters["RightArm"] == True:
            value = "Arms"

        elif parameters["LeftArm"] == True:
            value = "LArm"

        elif parameters["RightArm"] == True:
            value = "RArm"

        else:
            value = "None"

        return value

    def onTrackSound(self, input_ = "Head", parameters = {"LeftArm":False, "RigthArm":False}, parallel = False):

        new_parameters = {"use_body":input_, "use_arms":self.use_arms(parameters)}

        self.sensitivity = 0.9
        self.soundLocation.setParameter("Sensitivity", self.sensitivity)
        
        if self.running == True:
            self.onStop()
            print("stoping last tracking")
            if input_ != "None":
                self.onStart("Sound", new_parameters, parallel)   
        else:
            if input_ != "None":
                self.onStart("Sound", new_parameters, parallel)



    def onTrackPeople(self, input_ = "Head", parameters = {"LeftArm":False, "RigthArm":False}, parallel = False):

        new_parameters = {"use_body":input_, "use_arms":self.use_arms(parameters)}
        
        if self.running == True:
            self.onStop()
            print("stoping last tracking")
            if input_ != "None":
                self.onStart("People", new_parameters, parallel)   
        else:
            if input_ != "None":
                self.onStart("People", new_parameters, parallel)
            

    def onTrackRedBall(self, input_ = "Head", parameters = {"LeftArm":False, "RigthArm":False}, parallel = False):

        new_parameters = {"use_body":input_, "use_arms":self.use_arms(parameters)}
        
        if self.running == True:
            self.onStop()
            print("stoping last tracking")
            if input_ != "None":
                self.onStart("RedBall", new_parameters, parallel)   
        else:
            if input_ != "None":
                self.onStart("RedBall", new_parameters, parallel)


    def onWalkTowards(self, input_ = "People", parameters = {"LeftArm":False, "RigthArm":False}, parallel = False):

        new_parameters = {"use_body":"Move", "use_arms":self.use_arms(parameters)}
        
        if self.running == True:
            self.onStop()
            print("stoping last tracking")
            if input_ != "None":
                self.onStart(input_, new_parameters, parallel)   
        else:
            if input_ != "None":
                self.onStart(input_, new_parameters, parallel)
            
        
    def onUnload(self):
        if self.subscribeDone:
            self.memory.unsubscribeToEvent("ALTracker/TargetLost", self.name)
            self.memory.unsubscribeToEvent("ALTracker/TargetReached", self.name)
            self.subscribeDone = False

        if self.isRunning:
            self.tracker.setEffector("None")
            self.tracker.stopTracker()
            self.tracker.unregisterTarget(self.targetName)
            self.isRunning = False

    def onStart(self, input_, parameters = {"use_body":"Head", "use_arms":"None"}, parallel = False):
        self.running = True
        if not self.isRunning:

            mode = "Head"
            self.effector ="None"

            try:
                mode = parameters["use_body"]   # "WholeBody", "Move", "Head"
                self.effector = parameters["use_arms"]  #  "None", "Arms", "LArms", "RArms"
            except:
                pass

            self.targetName = input_   #"People", "Face", "RedBall", Sound
            self.distanceX = 0.3
            self.threshodX = 0.1
            self.distanceY = 0.0
            self.thresholdY = 0.1
            self.angleWz = 0.0
            self.thresholdWz = 0.3
            

            if self.subscribeDone:
                self.memory.unsubscribeToEvent("ALTracker/TargetLost", self.name)
                self.memory.unsubscribeToEvent("ALTracker/TargetReached", self.name)
                self.subscribeDone = False
            
            self.memory.subscribeToEvent("ALTracker/TargetLost", self.name, "onTargetLost")
            self.memory.subscribeToEvent("ALTracker/TargetReached", self.name, "onTargetReached")
            self.subscribeDone = True

            self.tracker.setEffector(self.effector)

            if input_ == "Sound":
                self.tracker.registerTarget(self.targetName, [self.soundDistance, self.confidence])
                self.tracker.setRelativePosition([-self.distance, 0.0, 0.0,
                                            self.thresholdX, 0.1, 0.3])
            else:
                #follow people
                peopleIds = []
                self.tracker.registerTarget(self.targetName, peopleIds)
                #follow ball

                if input_ == "RedBall":
                    diameter = 0.06000
                    self.tracker.registerTarget(self.targetName, diameter)


                    
                self.tracker.setRelativePosition([-self.distanceX, self.distanceY, self.angleWz,
                                                   self.thresholdX, self.thresholdY, self.thresholdWz])
            self.tracker.setMode(mode)
            self.tracker.track(self.targetName) # Start tracker
            print ("Tracking activated")
            self.isRunning = True

    def onStop(self):
        if self.isRunning:
            self.onUnload()
            print ("Tracking not activated")

    def onTargetLost(self, key, value, message):
        print ("Target lost")
        #self.targetLost()

    def onTargetReached(self, key, value, message):
        print ("Target reached")
        #self.targetReached()

    def onTargetChanged(self, key, value, message):
        if value == self.targetName and not self.subscribeDone:
            self.memory.subscribeToEvent("ALTracker/TargetLost", self.name, "onTargetLost")
            self.memory.subscribeToEvent("ALTracker/TargetReached", self.name, "onTargetReached")
            self.subscribeDone = True
        elif value != self.targetName and self.subscribeDone:
            self.memory.unsubscribeToEvent("ALTracker/TargetLost", self.name)
            self.memory.unsubscribeToEvent("ALTracker/TargetReached", self.name)
            self.subscribeDone = False



##
##robot_port = "9559"
##robot_name = "pepper"
##robot_ip = '192.168.2.34'
##
##pip   = robot_ip
##pport = int(robot_port)
##import nep_aldebaran
##say = nep_aldebaran.SayText(robot_ip, robot_port)
##
##
##track = Tracking(robot_ip,pip)
##times = 8
####say.onRun("Following people with head")
####track.onTrackPeople("Head", {"LeftArm":False, "RightArm":True})
####time.sleep(times)
####track.onTrackPeople("None", {"LeftArm":False, "RightArm":True})
####
####say.onRun("Following people with body")
####track.onTrackPeople("WholeBody", {"LeftArm":False, "RightArm":True})
####time.sleep(times)
####track.onTrackPeople("None", {"LeftArm":False, "RightArm":True})
####
####say.onRun("Following red ball with head")
####track.onTrackRedBall("Head", {"LeftArm":False, "RightArm":True})
####time.sleep(times)
####
####say.onRun("Following red ball with body")
####track.onTrackRedBall("WholeBody", {"LeftArm":False, "RightArm":True})
####time.sleep(times)
##
##say.onRun("Sound")
##track.onTrackSound("Head", {"LeftArm":False, "RightArm":True})
##time.sleep(50)
##
####say.onRun("Walking towards People")
####track.onWalkTowards("People", {"LeftArm":False, "RightArm":True})
####time.sleep(times)
####
####say.onRun("Walking towards RedBall")
####track.onWalkTowards("RedBall", {"LeftArm":False, "RightArm":True})
####time.sleep(times)
##
##track.onStop()





#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

class PeoplePerception(ALModule):

    def __init__(self, ip, port, memory, sharo, robot_name, th = 0.7):

        self.memory = memory
        self.robot = robot_name
        self.sharo = sharo
        self.port = 9559
        self.ip = ip

        self.name = "FaceEmotionEventListener"
        try:
            self.faceC = ALProxy("ALFaceCharacteristics",self.ip, self.port)
        except Exception as e:
            raise RuntimeError(str(e) + "Make sure you're not connected to a virtual robot." )        

        self.confidence = 0.35
        self.threshNeutralEmotion = self.confidence + 0.15
        self.threshHappyEmotion = self.confidence
        self.threshSurprisedEmotion = self.confidence + 0.05
        self.threshAngryEmotion = self.confidence + 0.2
        self.threshSadEmotion = self.confidence + 0.15
        self.emotions = ["neutral", "happy", "surprised", "angry", "sad"]
        self.counter = 0
        self.bIsRunning = False
        self.delayed = []
        self.errorMes = ""
        #print ( proxy_name + " success")
        self.timeOut = 1
        self.neutral = True
        self.happy = True
        self.surprised = True
        self.angry = True
        self.sad = True
        self.th = th

        
    def onStop(self):
            print("Emotion recognition stop")
            self.onUnload()
        
    def onUnload(self):
        self.counter = 0
        self.tProperties = [0,0,0,0,0]
        self.bIsRunning = False
        self.cancelDelays()


    def sendEmotion(self, emotion):
        data = {"primitive":"emotion", "input":{emotion:1}, "robot":self.robot}
        # print(emotion)
        self.sharo.send_json(data)

    def detectEmotion(self, ids, index):
        emotion = "none"
        try:

            #analyze only one face - Limitation of the NAO SDK?
            self.faceC.analyzeFaceCharacteristics(ids[0])
            time.sleep(0.2)
            properties = self.memory.getData("PeoplePerception/Person/"+str(ids[index])+"/ExpressionProperties")
            self.tProperties[0] = properties[0]
            self.tProperties[1] = properties[1]
            self.tProperties[2] = properties[2]
            self.tProperties[3] = properties[3]
            self.tProperties[4] = properties[4]

            recognized = [0,0,0,0,0]

            value = 0
            x = 0
            
            for i in range(len(recognized)):
                if (self.tProperties[i]>value and self.tProperties[i] > self.th ):
                    value = self.tProperties[i]
                    x = i

            if(x == 0 and not emotion == "neutral"):
                    self.sendEmotion("neutral")
                    
            elif(x == 1 and not emotion == "happy"):
                    self.sendEmotion("happy")
                
            elif(x == 2 and not emotion == "surprised"):
                    self.sendEmotion("surprised")
                    
            elif(x == 3 and not emotion == "angry"):
                    self.sendEmotion("angry")

            elif(x == 4 and not emotion == "sad"):
                    self.sendEmotion("sad")            
        except:
            time.sleep(.1)
                
        

    def onRun(self):
        print ("Emotion recognition started")
        self.tProperties = [0,0,0,0,0]
        self.bIsRunning = True
        time_not_seen = 10
        shirt_color = "white"
        face_detected = 0
        people_visible = 0
        distance = 1000
        alone = True
        time_th = 2
        
        while self.bIsRunning:

                #identify user
                ids = self.memory.getData("PeoplePerception/PeopleList")
                # print(ids)
                # print(len(ids))

                if len(ids) == 0:
                    self.errorMes = "No face detected"
                    alone = True

                    data = {"primitive":"human_detected", "input":{"no":1}, "robot":self.robot}
                    self.sharo.send_json(data)
                            
                elif len(ids) == 1:
                    try:
                        shirt_color = self.memory.getData("PeoplePerception/Person/"+str(ids[0])+"/ShirtColor")
                        time_not_seen = self.memory.getData("PeoplePerception/Person/"+str(ids[0])+"/NotSeenSince")
                        face_detected = self.memory.getData("PeoplePerception/Person/"+str(ids[0])+"/IsFaceDetected")
                        people_visible = self.memory.getData("PeoplePerception/Person/"+str(ids[0])+"/IsVisible")
                        distance = self.memory.getData("PeoplePerception/Person/"+str(ids[0])+"/Distance")
                        #print(time_not_seen)
                        
                        if time_not_seen > time_th:
                            if alone == False:
                                data = {"primitive":"people", "input":{"just_left":1}, "robot":self.robot}
                                self.sharo.send_json(data)
                                #print("alone")   
                            alone = True
                        else:
                            if alone == True:
                                data = {"primitive":"people", "input":{"just_arrived":1}, "robot":self.robot}
                                self.sharo.send_json(data)
                                #print("arrived")
                         
                            if face_detected==1:
                                self.detectEmotion(ids,0)

                            alone = False

                        data = {"primitive":"human_detected", "input":{"yes":1}, "robot":self.robot}
                        self.sharo.send_json(data)
                    

                    except:
                        time.sleep(.1)
                        
                elif len(ids) > 1:
                    data = {"primitive":"human_detected", "input":{"yes":1}, "robot":self.robot}
                    self.sharo.send_json(data)
                    
                time.sleep(1)

        
            

    def onTimeout(self):
        self.errorMes = "Timeout"
        self.onUnload()

    def cleanDelay(self, fut, fut_ref):
        self.delayed.remove(fut)

    def cancelDelays(self):
        cancel_list = list(self.delayed)
        for d in cancel_list:
            d.cancel()

##
##import nep
##import rize
##import nep_aldebaran
##
##robot_port = "9559"
##robot_name = "pepper"
##robot_ip = '192.168.11.47'
##middleware = "ZMQ"
##robot_info = {}
##type_robot = "pepper"
##
##pip   = robot_ip
##pport = int(robot_port)
##
##name_ = robot_name + "_sensors"
##node = nep.node(name_)  
##sharo = node.new_pub("/blackboard","json")
##
##sensors = nep_aldebaran.SensorEvents(robot_ip, int(robot_port))
##memory = sensors.getMemory()
##
##SpeechEventListener = PeoplePerception(robot_ip,pip, memory, sharo, robot_name)
##SpeechEventListener.onRun()
##time.sleep(5)
##SpeechEventListener.onStop()





#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""

class SensorEvents:
    def __init__(self,pip, pport = 9559):
        # We need this broker to be able to construct
        # # NAOqi modules and subscribe to other modules
        # The broker must stay alive until the program exists
        self.myBroker = ALBroker("myBroker",
            "0.0.0.0",   # listen to anyone
            0,           # find a free port and use it
            pip,         # parent broker IP
            int(pport))       # parent broker port

        self.memoryProxy = ALProxy("ALMemory", pip, int(pport))

    def getBroker(self):
        return self.myBroker
    
    def getMemory(self):
        return self.memoryProxy





#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""

class SayText:
    def __init__(self,ip, port = 9559):
        """ Define parameters for SayText function

            Parameters
            ----------

            ip : string
               IP value of the robot

        """
        self.port = int(port)
        self.ip = ip
        self.onLoad()

    def onLoad(self):
        try: 
            proxy_name ="ALTextToSpeech"
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            print ( proxy_name + " success")

            proxy_name ="ALAnimatedSpeech"
            self.animatedSpeechProxy = ALProxy(proxy_name,self.ip,self.port)
            print ( proxy_name + " success")
            
        except:
            print ( proxy_name + " error")
            return False

        return True
    
    def onSetLanguage(self, input_ = "", parameters = {}, parallel = False):
        
        try:
            self.proxy.setLanguage(input_)
        except:
            print ("Language " + str(input_) + " not avaliable in this robot")


    def onRun(self, input_ = "", parameters = {}, parallel = False):
        """ Run action primitive"""
        try:
            # Encode text
            textToSay = input_.encode("utf-8")

            # Set parameters
            if "velocity" in parameters:
                value = float(parameters["velocity"])
                if value > 350:
                    value = 350
                mapped_value = 50 +  value
                self.proxy.setParameter("speed", mapped_value)
                
            if "pitch" in parameters:
                value = float(parameters["pitch"])
                if(value > 100):
                    value = 100
                if(value < 0):
                    value = 0
                mapped_value =  .5 + value*1.26/100.0  #defualt = 1.3
                self.proxy.setParameter("pitchShift", mapped_value)
                
            if "language" in parameters:
                language = parameters["language"]
                try:
                    self.proxy.setLanguage(language)
                except:
                    print ("Language" + str(language) + "not avaliable in this robot")

            if "contextual" in parameters:
                if parameters["contextual"] == True:
                    if parallel:
                        self.animatedSpeechProxy.post.say(textToSay,{"bodyLanguageMode":"contextual"})
                    else:
                        self.animatedSpeechProxy.say(textToSay,{"bodyLanguageMode":"contextual"})
                else:
                    # Running mode
                    if parallel:
                        id_ = self.proxy.post.say(textToSay)
                    else:
                        id_ = self.proxy.say(textToSay)

                return 

            else:
                # Running mode
                if parallel:
                    id_ = self.proxy.post.say(textToSay)
                else:
                    id_ = self.proxy.say(textToSay)

            
            
                
        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(exc_type, fname, exc_tb.tb_lineno)

    def onStop(self):
        self.proxy.stopAll()
        






#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

# Template action funtion:


class Leds:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = int(port)
        self.onLoad()

    def onLoad(self):
        try: 
            proxy_name = "ALLeds"
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onSetColor(self, input_ = "", parameters = {}, parallel = False):   

        led_g = "AllLeds"
        if input_ == "eyes":
            led_g = "FaceLeds"
            
        r = float(parameters["R"])*.1
        g = float(parameters["G"])*.1
        b = float(parameters["B"])*.1

        try:
            time_to_change = float(parameters["time"])
        except Exception as e: #Defualt value .5 seconds
            time_to_change = .5
        
        if parallel:
            self.proxy.post.fadeRGB(led_g,r,g,b,time_to_change)
            print ("Led color changed in parallel")
        else:
            self.proxy.fadeRGB(led_g,r,g,b,time_to_change)
            print ("Led color changed")

    def onReset(self, input_ = "", parameters = {}, parallel = False):

        led_g = "AllLeds"
        if input_ == "eyes":
            led_g = "FaceLeds"
        
        if parallel:
            self.proxy.post.reset(led_g)
            print ("Led color changed in parallel")
        else:
            self.proxy.reset(led_g)
            print ("Led color changed")


##  --------- Test  ------------
## Change the robot ip only
            
##robot_port = "9559"
##robot_name = "pepper"
##robot_ip = '192.168.11.37'
##middleware = "nanomsg"
##pattern = "survey"
##
##leds = Leds(robot_ip, robot_port)
##leds.onSetColor("eyes", {"R":0,"G":0,"B":255})
##time.sleep(4)
##leds.onSetColor("all", {"R":255,"G":0,"B":255})
##time.sleep(4)
##leds.onReset()



#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""

class Volume:
    def __init__(self,ip, port = 9559):
        """ Define parameters for SayText function

            Parameters
            ----------

            ip : string
               IP value of the robot

        """
        self.port = int(port)
        self.ip = ip
        self.onLoad()

    def onLoad(self):
        try: 
            proxy_name ="ALAudioDevice"
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            print ( proxy_name + " success")
            
        except:
            print ( proxy_name + " error")
            return False

        return True
    

    def onSet(self, input_ = "", parameters = {}, parallel = False):
        """ Run action primitive"""
        try:
           
            value = float(input_)
            try:
                if(value > 100):
                    value = 100
                if(value < 0):
                    value = 0
                self.proxy.setOutputVolume(value) 
            except:
                pass
                
        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(exc_type, fname, exc_tb.tb_lineno)

    def onStop(self):
        self.proxy.stopAll()
        






#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import nep
import threading
import time
import sys



class SpeechRecognition(ALModule):
    def __init__(self, name, ip, sharo, robot_name):
        ALModule.__init__(self, name)

        self.sharo = sharo
        self.name = name
        print("event id name = " + self.name)
        self.ip = ip
        self.word_spotting = True # If false the robot will only understand exact expressions
        self.visualExpression = True # Led feedback
        self.port = 9559
        self.proxy_name = "ALSpeechRecognition"
        self.visualExpression = True
        self.wordSpotting = True
        self.sensitivity = 0.45
        self.robot_name = robot_name

        try:
            self.asr = ALProxy(self.proxy_name, ip, 9559)
            print ( self.proxy_name + " success")
            #self.onLoad(vocabulary)

        except():
            print ( self.proxy_name + " error")
            #self.memory.unsubscribeToEvent("WordRecognized")
            #self.onLoad(vocabulary)

        self.word = 0


    def onFeedback(self,value):
        self.asr.pause(True)
        self.asr.setAudioExpression(value)
        self.asr.setVisualExpression(value)
        self.asr.pause(False)

    def onSetSensitivity(self,value):
        self.sensitivity = value
        
    
    def onLoad(self,vocabulary, feedback, language):
        
        self.onFeedback(feedback)
        self.asr.pause(True)
        self.asr.setLanguage(language)
        self.asr.setVocabulary(vocabulary, False)

        self.asr.setVisualExpression(self.visualExpression)
        self.asr.pushContexts()
        self.asr.setVocabulary(vocabulary, self.wordSpotting )

        self.memory = ALProxy("ALMemory",self.ip, self.port)
        self.memory.subscribeToEvent("WordRecognized", self.name, "onWordRecognized")
        self.memory.subscribeToEvent("SpeechDetected", self.name, "onSpeechDetected")
        self.asr.pause(False)


    def onSetVocabulary(self, vocabulary, feedback, language):
        try:
            self.onLoad(vocabulary,feedback,language)
        except():
            self.onStop()
            try: 
                print ( self.proxy_name + "restarting ..")
                self.onLoad(vocabulary, feedback, language)
            except():
                print ( self.proxy_name + " error")


    def onSpeechDetected(self,  key, value, message):
        data = {"primitive":"speech", "input":{"speech":1}, "robot":self.robot_name}
        self.sharo.publish(data)
        

    def onWordRecognized(self, key, value, message):

        print "Key: ", key
        print "Value: " , value
        #print "Message: " , message
        word = ""
        if (len(value) > 1 and value[1] > self.sensitivity):

            if self.wordSpotting:
                values_rest = value[0].split("<...> ")
                values_final = values_rest[1].split(" <...>")
                word  = values_final[0]
            
            data = {"primitive":"word", "input":{word:1}, "robot":self.robot_name}
            self.sharo.publish(data)

    def onStop(self):
        self.memory.unsubscribeToEvent("WordRecognized",self.name)
        self.memory.unsubscribeToEvent("SpeechDetected",self.name)
        print("Event closed")


##
##robot_ip= "192.168.0.101"
##robot_port= "9559"
##
##pip   = robot_ip
##pport = int(robot_port)
##name_ = "nao" + "_speech"
##node = nep.node(name_)  
##sharo = node.new_pub("/blackboard","json")
##
##try:
##    # We need this broker to be able to construct
##    # NAOqi modules and subscribe to other modules
##    # The broker must stay alive until the program exists
##    myBroker = ALBroker("myBroker",
##       "0.0.0.0",   # listen to anyone
##       0,           # find a free port and use it
##       pip,         # parent broker IP
##       pport)       # parent broker port
##
##
##    SpeechEventListener = SpeechRecognition("SpeechEventListener",pip,sharo,"pepper")
##    SpeechEventListener.onSetVocabulary(["hello", "gym", "weather" , "stop", "good morning", "thank you", "yes", "no"], "English")
##    
##except Exception as e: 
##    print(e)
##    time.sleep(5)
##    sys.exit(0)
##
##time.sleep(30)
##       
##SpeechEventListener.onStop()
##myBroker.shutdown()
##sys.exit(0)




#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys
import copy

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""

class Bumpers:
    def __init__(self, memory, sharo, robot):
        self.memoryProxy = memory
        self.robot = robot
        self.sharo = sharo
        self.run = True
    
    def onRun(self):
        
        import copy
        old_proximity = "none"
        proximity = "none"
        error = False
        body = { "left": 0,  "right": 0,  "back": 0}
        old_body = copy.deepcopy(body)

        while self.run:

            try:
                r_value = memoryProxy.getData("Device/SubDeviceList/Platform/FrontRight/Bumper/Sensor/Value")
                l_value = memoryProxy.getData("Device/SubDeviceList/Platform/FrontLeft/Bumper/Sensor/Value")
                b_value = memoryProxy.getData("Device/SubDeviceList/Platform/Back/Bumper/Sensor/Value")


                if l_value > 0.4:
                    value = 1
                    if body["left"] != value:
                        data = {"primitive":"bumpers", "input":{"left":value}, "robot":self.robot}  
                        self.sharo.send_json(data)
                        body["left"] = value
                        print data

                else:
                    value = 0
                    if body["left"] != value:
                        data = {"primitive":"bumpers", "input":{"left":value}, "robot":self.robot} 
                        self.sharo.send_json(data)
                        body["left"] = value

                if r_value > 0.4:
                    value = 1
                    if body["right"] != value:
                        data = {"primitive":"bumpers", "input":{"right":value}, "robot":self.robot} 
                        self.sharo.send_json(data)
                        body["right"] = value
                        print data

                else:
                    value = 0
                    if body["right"] != value:
                        data = {"primitive":"bumpers", "input":{"right":value}, "robot":self.robot} 
                        self.sharo.send_json(data)
                        body["right"] = value

                if b_value > 0.4:
                    value = 1
                    if body["back"] != value:
                        data = {"primitive":"bumpers", "input":{"back":value}, "robot":self.robot} 
                        self.sharo.send_json(data)
                        body["back"] = value
                        print data

                else:
                    value = 0
                    if body["back"] != value:
                        data = {"primitive":"bumpers", "input":{"back":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        body["back"] = value

                old_body = copy.deepcopy(body)
            
            except:
                pass
                        
            time.sleep(.01)


#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys


class BehaviorManager:
    def __init__(self,ip, port = 9559):
        """ Define parameters for SayText function

            Parameters
            ----------

            ip : string
               IP value of the robot

        """
        self.port = int(port)
        self.ip = ip
        self.onLoad()

    def onLoad(self):
        try: 
            proxy_name ="ALBehaviorManager"
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")
            return False

        return True
    

    def onRun(self, input_ = "", parameters = {}, parallel = False):
        pass

    def onStop(self):
        self.proxy.stopAllBehaviors()
        






#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import qi
import time
import os
import sys
import math
import time
import almath
from tinydb import TinyDB, Query

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""

class BodyMove:
    def __init__(self,ip, port = 9559, robot = "pepper", animation_database_path ="animation_db.json"):
        """ Define parameters for SayText function

            Parameters
            ----------

            ip : string
               IP value of the robot

        """
        self.port = int(port)
        self.ip = ip
        self.robot = robot
        self.animation_database_path = animation_database_path
        print ("Animation path: " + str(animation_database_path))

        if self.robot == "nao":
            self.name_joints = ["LAnklePitch", "LAnkleRoll", "LHipRoll", "LHipYawPitch", "LKneePitch",\
                                    "RAnklePitch", "RAnkleRoll", "RHipRoll", "RHipYawPitch", "RKneePitch", \
                                    "LElbowRoll", "LElbowYaw", "LHand","LShoulderPitch", "LShoulderRoll", "LWristYaw", \
                                    "RElbowRoll", "RElbowYaw" , "RHand", "RShoulderPitch", "RShoulderRoll", "RWristYaw",\
                                    "HeadPitch", "HeadYaw"
                                    ]
        else:
            
            self.name_joints= ["HeadPitch","HeadYaw", "HipPitch", "HipRoll", "KneePitch", "LElbowRoll", "LElbowYaw",\
                                "LHand","LShoulderPitch", "LShoulderRoll", "LWristYaw", "RElbowRoll", "RElbowYaw" , "RHand", \
                                "RShoulderPitch", "RShoulderRoll", "RWristYaw"]
            
        self.onLoad()

    def onLoad(self):
        try: 
            proxy_name ="ALMotion"
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.positionErrorThresholdPos = 0.01
            self.positionErrorThresholdAng = 0.03

            self.session = qi.Session()
            self.session.connect("tcp://" + self.ip  + ":" + str(self.port))
            self.motion_service = self.session.service("ALMotion")

            if self.robot == "pepper":
                self.backgroundProxy = self.session.service("ALBackgroundMovement")
                print ( "ALBackgroundMovement" + " success")
            else:
                self.posture_service = self.session.service("ALRobotPosture")
                
                
            
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")
            return False

        return True

    def onPosture(self, input_ = "", parameters = {}, parallel = False):
        self.posture_service.goToPosture(input_, 1.0)
        

    def onBreathe(self, input_ = "", parameters = {}, parallel = False):
        if input_ == "on":
            self.motion_service.setEnabled(True)
        else:
            self.motion_service.setEnabled(False)


    def onRunMode(self, input_ = "", parameters = {}, parallel = False):
        if input_ == "rest" or input_ == "sleep":
            self.proxy.rest()
        elif input_ == "wake_up" or input_ == "wake up":
            # Wake up robot (Turn the motors on)
            self.proxy.wakeUp()
            if self.robot == "pepper":
                try:
                    self.backgroundProxy.setEnabled(True)
                    #self.motion_service.setBreathEnabled("Body", True)
                except:
                    print("Error blakground movement, this is a pepper robot?")
            else:
                try:
                    #self.backgroundProxy.setEnabled(True)
                    self.motion_service.setBreathEnabled("Body", True)
                except:
                    print("Error setBreathEnabled")
                



    def onOpenHand(self,input_ = "", parameters = {}, parallel = False):

        all_ = False

        if "all" in parameters:
            all_ = parameters["all"]

             
        if input_ == "right":

            if all_: 
                self.motion_service.setAngles("RHand", 0.98, 0.1)
            else: 
                self.motion_service.setAngles("RHand", 0.6, 0.1)

            
        elif input_ == "left":
        
            if all_: 
                self.motion_service.setAngles("LHand", 0.98, 0.1)
            else: 
                self.motion_service.setAngles("LHand", 0.6, 0.1)

        else:
            if all_: 
                self.motion_service.setAngles("LHand", 0.98, 0.1)
                self.motion_service.setAngles("RHand", 0.98, 0.1)
               
            else: 
                self.motion_service.setAngles("LHand", 0.6, 0.1)
                self.motion_service.setAngles("RHand", 0.6, 0.1)
                



    def onCloseHand(self,input_ = "", parameters = {}, parallel = False):

        if input_ == "right":
            self.motion_service.setAngles("RHand", 0.02, 0.1)
            
        elif input_ == "left":
            self.motion_service.setAngles("LHand", 0.02, 0.1)

        else:
            self.motion_service.setAngles("LHand", 0.02, 0.1)
            self.motion_service.setAngles("RHand", 0.02, 0.1)

    def onWalk(self, input_ = "", parameters = {}, parallel = False):

        try:
            if input_ == "backwards":
                x =  -1*float(parameters["meters"])
                y = float(0)
                Theta = float(0)
                self.onMotionPosition(x,y,Theta,parallel)

            else:
                x =  float(parameters["meters"])
                y = float(0)
                Theta = float(0)
                self.onMotionPosition(x,y,Theta, parallel)

        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(exc_type, fname, exc_tb.tb_lineno)


    def onTurn(self, input_ = "", parameters = {}, parallel = False):

        try:
            if input_ == "left":
                x = float(0)
                y = float(0)
                Theta = float(parameters["degrees"])
                self.onMotionPosition(x,y,Theta,parallel)

            else:
                x = float(0) 
                y = float(0)
                Theta = -1*float(parameters["degrees"])
                self.onMotionPosition(x,y,Theta, parallel)

        except Exception as e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print(exc_type, fname, exc_tb.tb_lineno)

            


    def onMotionPosition(self,x,y,Theta,parallel):

        motion_service = self.motion_service

        #####################
        ## Enable arms control by move algorithm
        #####################
        motion_service.setMoveArmsEnabled(True, True)
        
        # motion_service.setMoveArmsEnabled(False, False)

        #####################
        ## FOOT CONTACT PROTECTION
        #####################
        # motion_service.setMotionConfig([["ENABLE_FOOT_CONTACT_PROTECTION",False]])
        motion_service.setMotionConfig([["ENABLE_FOOT_CONTACT_PROTECTION", True]])

        #####################
        ## get robot position before move
        #####################
        initRobotPosition = almath.Pose2D(motion_service.getRobotPosition(False))
        motion_service.setMoveArmsEnabled(False, False)


        X = float(x)
        Y = float(y)
        Theta = Theta*math.pi/180

        if  parallel == True:
            id = motion_service.post.moveTo(X, Y, Theta)
            # wait is useful because with _async moveTo is not blocking function
            motion_service.wait(id,0)

            
        else:   
            motion_service.moveTo(X, Y, Theta, _async=True)
            # wait is useful because with _async moveTo is not blocking function
            motion_service.waitUntilMoveIsFinished()

        #####################
        ## get robot position after move
        #####################
        endRobotPosition = almath.Pose2D(motion_service.getRobotPosition(False))

        #####################
        ## compute and print the robot motion
        #####################
        robotMove = almath.pose2DInverse(initRobotPosition)*endRobotPosition
        # return an angle between ]-PI, PI]
        robotMove.theta = almath.modulo2PI(robotMove.theta)
        print ("Robot Move:", robotMove)

        
    
    def onMove(self, input_ = "", parameters = {}, parallel = False):

        if input_ == "position":
            try:
                x = float(parameters["x"])
                y = float(parameters["y"])
                Theta = float(parameters["angle"])
                motion_service = self.self.motion_service
                self.__onMotionPosition(self,x,y,Theta)
                
            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print(exc_type, fname, exc_tb.tb_lineno)
            
        elif input_ == "velocity":
            x = float(parameters["x"])
            y = float(parameters["y"])
            theta = float(parameters["angle"])

            if not parallel:

                enableArms = True
                self.proxy.setMoveArmsEnabled(enableArms, enableArms)

            
                if "frequency" in parameters:
                    
                    frequency = float(parameters["frequency"])
                else:
                    frequency = 0.5
                    
                if self.robot == "nao":
                    x = float(parameters["x"])
                    y = float(parameters["y"])
                    theta = float(parameters["angle"])
                    frequency = 0.5
                    #frequency = parameters["frequency"]
                    self.proxy.setWalkTargetVelocity(x,y,theta,frequency)
                else:
                    self.proxy.moveToward(x, y, theta) 
            else:

                
                enableArms = True
                self.setMoveArmsEnabled(enableArms, enableArms)

            
                if "frequency" in parameters:
                    
                    frequency = float(parameters["frequency"])
                else:
                    frequency = 0.5
                    
                if self.robot == "nao":
                    x = float(parameters["x"])
                    y = float(parameters["y"])
                    theta = float(parameters["angle"])
                    frequency = 0.5
                    #frequency = parameters["frequency"]
                    self.proxy.post.setWalkTargetVelocity(x,y,theta,frequency)
                else:
                    self.proxy.post.moveToward(x, y, theta) 



    def onRunAnimation(self, input_ = "", parameters = {}, parallel = False):
        """ Run action primitive"""
        
        try:
            # Animation trayectory parameters              
            names = list()
            times = list()
            keys = list()
            flip = False
            reverse = False
            
            if "flip" in parameters:
                flip = parameters["flip"]
            if "reverse" in parameters:
                reverse = parameters["reverse"]
            # Get animation from database
            success, names, times, keys = self._animation_query(input_, flip, reverse)
            self.name_joints = names

            # If valid animation
            if success:

                if "time" in parameters:
                    # Set animation duration
                    times =  self._change_animation_duration(times,float(parameters["time"]))

                try:
                    if parallel:
                        self.proxy.post.angleInterpolation(names, keys, times, True)
                    else:
                        self.proxy.angleInterpolation(names, keys, times, True)
                        
                except BaseException, err:
                    print (err)
            else:
                print ('Animation execution error')
        except BaseException, err:
            print ('Animation not found in database')
                


    def onStopAnimation(self):
        #self.proxy.stopMove()
        self.proxy.killAll();


    # ------------------------------------ Animation parametrization ----------------------------------------------- 

    # Invert movement from left to rigth
    def __invert_side(self,old_keys,old_names,old_times):
        joint2negative = ["HeadYaw","HipRoll"]
        keys = old_keys
        names = old_names
        times = old_times

        for joint_name in joint2negative:
            if joint_name in names:
                index_number = names.index(joint_name)
                temp_keys = keys[index_number]
                keys[index_number] = map(lambda x: x*(-1), temp_keys)


        joints2flip = [["LElbowRoll","RElbowRoll"], ["LElbowYaw", "RElbowYaw"],["LShoulderPitch","RShoulderPitch"], ["LShoulderRoll", "RShoulderRoll"], ["LWristYaw", "RWristYaw"]]
            
        for joint_tuple in joints2flip:
            if joint_tuple[0] in names and joint_tuple[1] in names:
                index_number1 = names.index(joint_tuple[0])
                index_number2 = names.index(joint_tuple[1])

                print joint_tuple
                print keys[index_number1]
                print keys[index_number2]
                
                temp_keys1 = keys[index_number1]
                temp_keys2 = keys[index_number2]

                if joint_tuple[0] == "LShoulderPitch":
                    pass
                else:
                    temp_keys1 = map(lambda x: x*(-1), temp_keys1)
                    temp_keys2 = map(lambda y: y*(-1), temp_keys2)
                
                temp_times1 = times[index_number1]
                temp_times2 = times[index_number2]

                keys[index_number1] = temp_keys2
                keys[index_number2] = temp_keys1
                times[index_number1] = temp_times2
                times[index_number2] = temp_times1

                print keys[index_number1]
                print keys[index_number2]

            print  


        return keys,names,times

    # Invert trayectory points
    def __invert_time(self,times):
        # Invert diference between motions
        time_dif = []
        new_times = [times[0]]
        time_len = len(times)
        for t in range (time_len-1):
            time_dif.append(times[t+1]- times[t])

        new_time_dif = time_dif[::-1]
        
        for t in range (time_len-1):
            new_times.append(new_times[t]+new_time_dif[t])
        return new_times

    # Get Animation from database
    def _animation_query(self,name_animation, flip = False, reverse = False): #1
        """ Get the animation parameters (joint names, times and joint values) of a specific animation from a database"""
        db = TinyDB(self.animation_database_path)
        q = Query()
        try:
            s = db.search(q.animation == name_animation)
        except:
            print (" ERROR: Animation not found in database")
            return False,[0],[0],[0]

        names = [x.encode('UTF8') for x in s[0]['names']] # Name of the joint
        times =  s[0]['times']                            # Time index
        keys =  s[0]['keys']                              # Joint values

        if flip == True:
            keys,names,times = self.__invert_side(keys,names,times)

        if reverse == True and len(times) > 1 :
            i = 0
            for k in keys:
                temp = k[::-1]
                keys[i] = temp
                i = i + 1
            i = 0
            for time in times:
                times[i] = self.__invert_time(time)
                i = i + 1                
        return True, names, times, keys

        
    # Set animatio duration
    def _change_animation_duration(self, time_list, new_max):
        """ Change the timeline values to other specified by the user
        """
        
        l = len(time_list)
        time_base = time_list[0]
        max_time = float(time_base[-1])
        factor = float(new_max)/max_time
        for lista in time_list:
            for index, item in enumerate(lista):
                lista[index] = lista[index]*factor
        
        return time_list
        






#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys
import copy

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""


class Touch:

    def __init__(self, memory, sharo, robot_name, type_robot = "pepper", debug = False):
        self.memoryProxy = memory
        self.robot = robot_name
        self.sharo = sharo
        self.run = True
        self.type_robot = type_robot
        self.debug = debug

    def onRun(self):
        import copy
        
        body = { "left_hand": 0,  "right_hand": 0,  "head": 0}
        old_body = copy.deepcopy(body)
        value_touched = 0.4

        while self.run:
            
            try:

                if self.type_robot=="pepper":

                    self.Lhand_touched = self.memoryProxy.getData("Device/SubDeviceList/LHand/Touch/Back/Sensor/Value")
                    self.Rhand_touched = self.memoryProxy.getData("Device/SubDeviceList/RHand/Touch/Back/Sensor/Value")
                    
                else:

                    Lhand_touched1 = self.memoryProxy.getData("Device/SubDeviceList/LHand/Touch/Back/Sensor/Value")
                    Lhand_touched2 = self.memoryProxy.getData("Device/SubDeviceList/LHand/Touch/Left/Sensor/Value")
                    Lhand_touched3 = self.memoryProxy.getData("Device/SubDeviceList/LHand/Touch/Right/Sensor/Value")
                    self.Lhand_touched = max([Lhand_touched1,Lhand_touched2,Lhand_touched3])
                    
                    Rhand_touched1 = self.memoryProxy.getData("Device/SubDeviceList/RHand/Touch/Back/Sensor/Value")
                    Rhand_touched2 = self.memoryProxy.getData("Device/SubDeviceList/RHand/Touch/Left/Sensor/Value")
                    Rhand_touched3 = self.memoryProxy.getData("Device/SubDeviceList/RHand/Touch/Right/Sensor/Value")
                    self.Rhand_touched =  max(Rhand_touched1,Rhand_touched2,Rhand_touched3)
                    
                
                self.Head_touched = self.memoryProxy.getData("Device/SubDeviceList/Head/Touch/Middle/Sensor/Value")
                self.Head_touched_front = self.memoryProxy.getData("Device/SubDeviceList/Head/Touch/Front/Sensor/Value")
                self.Head_touched_rear = self.memoryProxy.getData("Device/SubDeviceList/Head/Touch/Rear/Sensor/Value")

                if self.Lhand_touched > value_touched:
                    value = 1
                    if body["left_hand"] != value:
                        data = {"primitive":"touched", "input":{"left_hand":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        body["left_hand"] = value
                        if self.debug:
                            print (data)
                else:
                    value = 0
                    if body["left_hand"] != value:
                        data = {"primitive":"touched", "input":{"left_hand":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        body["left_hand"] = value
                        if self.debug:
                            print (data)

                if self.Rhand_touched > value_touched:
                    value = 1
                    if body["right_hand"] != value:
                        data = {"primitive":"touched", "input":{"right_hand":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        if self.debug:
                            print (data)
                else:
                    value = 0
                    if body["right_hand"] != value:
                        data = {"primitive":"touched", "input":{"right_hand":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        body["right_hand"] = value
                        if self.debug:
                            print (data)
                    
                if self.Head_touched > value_touched or self.Head_touched_front > value_touched or self.Head_touched_rear > value_touched:
                    value = 1
                    if body["head"] != value:
                        data = {"primitive":"touched", "input":{"head":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        body["head"] = value
                        if self.debug:
                            print (data)
                else:
                    value = 0
                    if body["head"] != value:
                        data = {"primitive":"touched", "input":{"head":value}, "robot":self.robot}
                        self.sharo.send_json(data)
                        body["head"] = value
                        if self.debug:
                            print (data)
                            
                old_body = copy.deepcopy(body)
                
            except Exception as e:
                exc_type, exc_obj, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print(exc_type, fname, exc_tb.tb_lineno)
                
            time.sleep(.01)



from SayText import*
from BodyMove import*
from AutonomusLife import*
from BodyMove import*
from BehaviorManager import*
from Volume import*
from SpeechRecognition import*
from Leds import*
from Audio import*
from Tablet import*
from SensorEvents import*
from Touch import*
from Sonars import*
from Bumpers import*
from Tracking import*
from PeoplePerception import*












#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys

# Template action funtion:
"""

class name:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = port

    def onLoad(self):
        try: 
            proxy_name "AL.."
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

    #onRun for action, onInput for peception.
    def onRun(self, input_ = "", parameters = {}, parallel = "false"):   

    def onStop(self, input_ = "", parameters = {}, parallel = "false"):


"""
# Template module:
"""

class NameModule(ALModule):
    def __init__(self, name, robot, ip  port = 9559):
        ALModule.__init__(self, name)
        self.name = name
        self.robot = robot
        self.ip = ip
        self.port = port
        try: 
            proxy_name = "AL.."
            self.proxy = ALProxy(proxy_name,self.ip,self.port)
            self.memory = ALProxy("ALMemory",self.ip, self.port)
            print ( proxy_name + " success")

            try:
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")
            except():
                self.memory.unsubscribeToEvent(EventName, self.name)
                self.memory.subscribeToEvent(EventName, self.name, "EventListener")

        except:
            print ( proxy_name + " error")

    def EventListener(self, key, value, message):


"""


class Sonars:
    def __init__(self, memory, sharo, robot, close = 0.5, middle = 1.0 , far = 1.4 , varyFar = 1.8):
        self.memoryProxy = memory
        self.robot = robot
        self.sharo = sharo
        self.run = True
        self.close = close
        self.middle = middle
        self.far = far
        self.varyFar = varyFar
    
    def onRun(self):
        
        old_proximity = "none"
        proximity = "none"
        error = False

       

        while self.run:

            try:
                front_value = self.memoryProxy.getData("Device/SubDeviceList/Platform/Front/Sonar/Sensor/Value")
                back_value = self.memoryProxy.getData("Device/SubDeviceList/Platform/Back/Sonar/Sensor/Value")


                if front_value < self.close:
                    proximity = "close"
                elif front_value < self.middle  and front_value > self.close:
                    proximity = "middle"
                elif front_value < self.far and front_value > self.middle : 
                    proximity = "far"
                elif front_value > self.varyFar:
                    proximity = "very far"
               
                if old_proximity == proximity:
                    pass
                else:
                    print ("Human distance: " + proximity)
                    data = {"primitive":"human_distance", "input":{proximity:1}, "robot":self.robot} 
                    self.sharo.send_json(data)
            except:
                if not error:
                    print ("Sonar problem")
                    error = True
                else:
                    pass
            old_proximity = proximity
            time.sleep(.01)


#!/usr/bin/env python
# -*- encoding: UTF-8 -*-

# Luis Enrique Coronado Zuniga

# You are free to use, change, or redistribute the code in any way you wish
# but please maintain the name of the original author.
# This code comes with no warranty of any kind.

from naoqi import ALProxy
from naoqi import ALBroker
from naoqi import ALModule
import time
import os
import sys
import datetime
import multiprocessing

class Tablet:
    def __init__(self,ip,port=9559):
        self.ip = ip
        self.port = int(port)
        self.onLoad()

    def onLoad(self):
        try: 
            proxy_name = "ALTabletService"
            self.proxy = ALProxy(proxy_name, self.ip, self.port)
            print ( proxy_name + " success")
        except:
            print ( proxy_name + " error")

        try:
            self.photoCaptureProxy =  ALProxy("ALPhotoCapture", self.ip, self.port)
            print ("ALPhotoCapture success")
        except:
            print ("ALPhotoCapture error")
            pass

    def showImage(self, input_, parameters = {}, parallel = False):
        self.proxy.showImage("http://198.18.0.1/apps/images/"+input_+".jpg")
        
    def showWebImage(self, input_, parameters = {}, parallel = False):
        self.proxy.showImageNoCache(input_)

    def takePhoto(self, input_, parameters = {}, parallel = False):
        
        camera = "Top"
        self.photoCaptureProxy.setResolution(1)
        self.photoCaptureProxy.setCameraID(1) # camera ='Top': 0 or 'Bottom': 1
        self.photoCaptureProxy.setPictureFormat("jpg")
        self.photoCaptureProxy.takePicture("/var/persistent/home/nao/.local/share/PackageManager/apps/images/html/", input_)
        time.sleep(.5)

    def takeNewImageShow(self, input_ = "", parameters = {}, parallel = False):
        x = datetime.datetime.now()
        self.image_name = "img_" + x.strftime("%m%d%H%M%S")
        p = multiprocessing.Process(target=self.takePhoto(input_))
        p.start()
        # Wait for 10 seconds or until process finishes
        p.join(2)
        if p.is_alive():
            p.terminate()
            p.join()
        self.proxy.stopVideo()
        self.proxy.hideImage()
        time.sleep(10)
        print self.image_name
        self.proxy.showImage("http://198.18.0.1/apps/images/"+self.image_name+".jpg")
        time.sleep(2)

    def takeImage(self, input_ = "", parameters = {}, parallel = False):
        x = datetime.datetime.now()
        self.image_name = "photo_show"
        p = multiprocessing.Process(target=self.takePhoto(input_))
        p.start()
        # Wait for 10 seconds or until process finishes
        p.join(2)
        if p.is_alive():
            p.terminate()
            p.join()
        self.proxy.stopVideo()
        self.proxy.hideImage()
        time.sleep(10)
        print self.image_name
                             

    def showWeb(self, input_, parameters = {}, parallel = False):
        self.proxy.hideImage()
        self.proxy.stopVideo()
        self.proxy.loadUrl(input_)
        self.proxy.showWebview()

    def showVideo(self, input_, parameters = {}, parallel = False):
        self.proxy.hideImage()
        self.proxy.stopVideo()
        self.proxy.enableWifi()
        path = "http://198.18.0.1/apps/videos/" + str(input_) + ".mp4"
        self.proxy.playVideo(path)

    def onReset(self, input_ = "", parameters = {}, parallel = False):
        self.proxy.resetTablet()

    def onHideImage(self, input_ = "", parameters = {}, parallel = False):
        self.proxy.hideImage()
        
        

        
##robot_port = "9559"
##robot_name = "pepper"
##robot_ip = '192.168.11.3'
##import nep
##
##url = "http://" + nep.getMyIP() + ":3000/"
##
##tablet = Tablet(robot_ip, robot_port)
###tablet.showImage("gvlab")
###time.sleep(4)
###tablet.showWebImage("http://vignette.wikia.nocookie.net/animevice/images/d/d3/GochiUsa_2_End_Card_12.png")
###time.sleep(4)
####tablet.showWeb(url)
####time.sleep(10)
###tablet.showVideo("meditation")
###time.sleep(10)
###tablet.onReset()
###time.sleep(4)
###tablet.takeImage("img_0605190205")
##tablet.showImage("img_0605190205")
###time.sleep(2)
###tablet.takeNewImageShow()
###time.sleep(5)
###tablet.showImage("gvlab")


